@@ -16,13 +16,14 @@ import urlparse
 import feedcircuit
 import urllib
 import webbrowser
+import glob
 
 gtk.gdk.threads_init()
 
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.4", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.5", False)
 except ImportError:
     pass
 
@@ -61,12 +62,9 @@ except ImportError:
 
 
 def create_config_dir():
-    path = os.path.expanduser("~/.config")
+    path = os.path.expanduser("~/.config/feedcircuit")
     if not os.path.exists(path):
-        os.mkdir(path)
-    path = os.path.join(path, "feedcircuit")
-    if not os.path.exists(path):
-        os.mkdir(path)
+        os.makedirs(path)
     return path
 
 
@@ -135,12 +133,13 @@ class FeedConfig:
     mode_inline = 2
 
 
-    def __init__(self, url, name, mode, days_to_keep,
+    def __init__(self, cache_path, url, name, mode, days_to_keep,
         include, exclude, ignore_print_version, icon_filename,
         message_count, as_is, content_tag, print_version_link, reformatter_url, id = None):
         self.id = id
         if not self.id:
             self.id = uuid.uuid1().hex
+        self.cache_path = cache_path
         self.url = url
         self.name = name
         self.mode = mode
@@ -162,7 +161,7 @@ class FeedConfig:
     def get_icon(self):
         if not self.icon:
             if self.icon_filename:
-                icon_filepath = os.path.join(create_config_dir(), self.icon_filename)
+                icon_filepath = os.path.join(self.cache_path, self.icon_filename)
                 if os.path.exists(icon_filepath):
                     self.icon = gtk.gdk.pixbuf_new_from_file(icon_filepath)
                     if self.icon.get_height() > 32:
@@ -172,7 +171,7 @@ class FeedConfig:
 
 
     def get_time(self):
-        filename = os.path.join(create_config_dir(), self.id + ".html")
+        filename = os.path.join(self.cache_path, self.id + ".html")
         if os.path.exists(filename):
             return datetime.fromtimestamp(os.path.getmtime(filename)).strftime("%c")
         return ""
@@ -284,24 +283,27 @@ class FeedListModel(gtk.GenericTreeModel):
     def load_config(self):
         parser = ConfigParserEx()
         parser.read([os.path.join(self.config_path, "config")])
+        self.cache_path = parser.get_default("global", "cache path", self.config_path)
         for section in parser.sections():
-            feed_config = FeedConfig(
-                parser.get(section, "url"),
-                parser.get(section, "name"),
-                parser.getint_default(section, "mode", FeedConfig.mode_ommit),
-                parser.getint_default(section, "days", 7),
-                parser.get_default(section, "include", ""),
-                parser.get_default(section, "exclude", ""),
-                parser.getboolean_default(section, "ignore print version", False),
-                parser.get_default(section, "icon", ""),
-                parser.getint_default(section, "message count", 0),
-                parser.getboolean_default(section, "as is", False),
-                parser.get_default(section, "content tag", ""),
-                parser.get_default(section, "print version link", ""),
-                parser.get_default(section, "reformatter url", ""),
-                section)
-            feed_config.ord_num = parser.getint_default(section, "order number", 0)
-            self.list.append(feed_config)
+            if section != "global":
+                feed_config = FeedConfig(
+                    self.cache_path,
+                    parser.get(section, "url"),
+                    parser.get(section, "name"),
+                    parser.getint_default(section, "mode", FeedConfig.mode_ommit),
+                    parser.getint_default(section, "days", 7),
+                    parser.get_default(section, "include", ""),
+                    parser.get_default(section, "exclude", ""),
+                    parser.getboolean_default(section, "ignore print version", False),
+                    parser.get_default(section, "icon", ""),
+                    parser.getint_default(section, "message count", 0),
+                    parser.getboolean_default(section, "as is", False),
+                    parser.get_default(section, "content tag", ""),
+                    parser.get_default(section, "print version link", ""),
+                    parser.get_default(section, "reformatter url", ""),
+                    section)
+                feed_config.ord_num = parser.getint_default(section, "order number", 0)
+                self.list.append(feed_config)
         self.list.sort(lambda f1, f2: f1.ord_num.__cmp__(f2.ord_num))
         self.rebuild_map()
 
@@ -310,6 +312,8 @@ class FeedListModel(gtk.GenericTreeModel):
         f = open(os.path.join(self.config_path, "config"), "w")
         try:
             parser = ConfigParserEx()
+            parser.add_section("global")
+            parser.set("global", "cache path", self.cache_path)
             i = 0
             for feed_config in self.list:
                 parser.add_section(feed_config.id)
@@ -361,6 +365,47 @@ class FeedListModel(gtk.GenericTreeModel):
         self.save_config()
 
 
+    def update_cache_path(self, cache_path):
+        self.cache_path = cache_path
+        for feed_config in self.list:
+            feed_config.cache_path = cache_path
+
+
+class PreferencesWindow(gtk.Dialog):
+
+
+    def __init__(self, parent):
+        gtk.Dialog.__init__(self, None, parent,
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
+             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        self.set_title(_("Preferences"))
+        hbox = gtk.HBox()
+        self.path_entry = gtk.Entry()
+        self.path_entry.set_width_chars(24)
+        hbox.pack_start(self.path_entry, expand = True)
+        self.choose_button = gtk.ToolButton(stock_id = gtk.STOCK_DIRECTORY)
+        self.choose_button.connect("clicked", self.choose_button_clicked)
+        hbox.pack_start(self.choose_button, expand = False)
+        table = gtk.Table(rows = 1, columns = 2)
+        texts = [_("Cache path")]
+        for i in range(0, len(texts)):
+            label = gtk.Label(texts[i])
+            table.attach(label, 0, 1, i, i + 1, xpadding = 3, ypadding = 3)
+        table.attach(hbox, 1, 2, 0, 1, xpadding = 3, ypadding = 3)
+        self.vbox.add(table)
+        self.show_all()
+
+
+    def choose_button_clicked(self, source):
+        dlg = gtk.FileChooserDialog(title = _("Select folder"), parent = self, action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
+            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+        dlg.set_filename(self.path_entry.get_text())
+        if dlg.run() == gtk.RESPONSE_ACCEPT:
+            self.path_entry.set_text(dlg.get_filename())
+        dlg.destroy()
+
+
 class AddWindow(gtk.Dialog):
 
 
@@ -416,14 +461,14 @@ class ContentWindow(gtk.Dialog):
              gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
         self.set_title(_("Content settings"))
         self.as_is_button = gtk.CheckButton()
-        self.as_is_button.connect("toggled", self.as_is_changed)
+        self.as_is_button.connect("toggled", self.something_changed)
         self.content_tag_entry = gtk.Entry()
         self.content_tag_entry.set_width_chars(24)
-        self.content_tag_entry.connect("changed", self.content_tag_entry_changed)
+        self.content_tag_entry.connect("changed", self.something_changed)
         self.reformatter_url_entry = gtk.Entry()
-        self.reformatter_url_entry.connect("changed", self.reformatter_url_entry_changed)
+        self.reformatter_url_entry.connect("changed", self.something_changed)
         self.ignore_print_version_button = gtk.CheckButton()
-        self.ignore_print_version_button.connect("toggled", self.ignore_print_version_changed)
+        self.ignore_print_version_button.connect("toggled", self.something_changed)
         self.print_version_link_entry = gtk.Entry()
         table = gtk.Table(rows = 5, columns = 2)
         texts = [_("Use as is"), _("Content tag"), _("Reformatting service url"),
@@ -441,27 +486,15 @@ class ContentWindow(gtk.Dialog):
         self.show_all()
 
 
-    def as_is_changed(self, source):
+    def something_changed(self, source):
         self.ignore_print_version_button.set_sensitive(not self.as_is_button.get_active())
-        self.print_version_link_entry.set_sensitive(not self.as_is_button.get_active())
-        self.content_tag_entry.set_sensitive(not self.as_is_button.get_active())
-        self.reformatter_url_entry.set_sensitive(not self.as_is_button.get_active())
-
-
-    def ignore_print_version_changed(self, source):
-        self.print_version_link_entry.set_sensitive(not self.ignore_print_version_button.get_active())
-
-
-    def content_tag_entry_changed(self, source):
-        self.ignore_print_version_button.set_sensitive(len(self.content_tag_entry.get_text()) == 0)
-        self.print_version_link_entry.set_sensitive(len(self.content_tag_entry.get_text()) == 0)
-        self.reformatter_url_entry.set_sensitive(len(self.content_tag_entry.get_text()) == 0)
-
-
-    def reformatter_url_entry_changed(self, source):
-        self.ignore_print_version_button.set_sensitive(len(self.reformatter_url_entry.get_text()) == 0)
-        self.print_version_link_entry.set_sensitive(len(self.reformatter_url_entry.get_text()) == 0)
-        self.content_tag_entry.set_sensitive(len(self.reformatter_url_entry.get_text()) == 0)
+        self.print_version_link_entry.set_sensitive(
+            not self.as_is_button.get_active() and
+            not self.ignore_print_version_button.get_active())
+        self.content_tag_entry.set_sensitive(
+            not self.as_is_button.get_active() and len(self.reformatter_url_entry.get_text()) == 0)
+        self.reformatter_url_entry.set_sensitive(
+            not self.as_is_button.get_active() and len(self.content_tag_entry.get_text()) == 0)
 
 
 class EditWindow(gtk.Dialog):
@@ -576,7 +609,6 @@ class Program(hildon.Program):
         self.__window_in_fullscreen = False
         self.add_window(self.__window)
        
-        self.__config_path = create_config_dir()
         self.__abort_lock = Lock()
         self.__update_lock = Lock()
         
@@ -591,6 +623,8 @@ class Program(hildon.Program):
         self.append_action(menu, None, None, gtk.STOCK_EDIT, self.edit)
         self.append_action(menu, None, None, gtk.STOCK_DELETE, self.remove)
         menu.append(gtk.SeparatorMenuItem())
+        self.append_action(menu, None, None, gtk.STOCK_PREFERENCES, self.preferences)
+        menu.append(gtk.SeparatorMenuItem())
         self.append_action(menu, None, None, gtk.STOCK_ABOUT, self.about)
         self.append_action(menu, None, None, gtk.STOCK_QUIT, self.stop)
         self.__progress_label = gtk.Label()
@@ -608,7 +642,7 @@ class Program(hildon.Program):
         self.__window.set_menu(menu)
         
         self.__feeds = {}
-        self.__model = FeedListModel(self.__config_path)
+        self.__model = FeedListModel(create_config_dir())
         self.__view = gtk.TreeView(self.__model)
         self.__view.set_rubber_banding(True)
         self.__view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
@@ -630,7 +664,10 @@ class Program(hildon.Program):
 
         self.__view.connect("row_activated", self.row_activated)
 
-        self.__window.add(self.__view)
+        sw = gtk.ScrolledWindow()
+        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        sw.add(self.__view)
+        self.__window.add(sw)
         self.__window.add_toolbar(self.__toolbar)
 
 
@@ -672,7 +709,7 @@ class Program(hildon.Program):
 
 
     def open_feed(self, feed_config):
-        path = os.path.join(self.__config_path, feed_config.id + ".html")
+        path = os.path.join(self.__model.cache_path, feed_config.id + ".html")
         if os.path.exists(path):
             webbrowser.open(path)
             feed_config.message_count = 0
@@ -747,7 +784,7 @@ class Program(hildon.Program):
             for feed_config in feed_configs:
                 i += 1
                 self.__progress_info = (i, len(feed_configs))
-                feed = feedcircuit.Feed(url = feed_config.url, path = self.__config_path, cache_path = ".cache",
+                feed = feedcircuit.Feed(url = feed_config.url, path = self.__model.cache_path, cache_path = ".cache",
                     title = feed_config.name, filename = feed_config.id + ".html",
                     callback = self.callback, try_use_print_version = not feed_config.ignore_print_version,
                     delete_before_date = date.today() - timedelta(feed_config.days_to_keep),
@@ -811,17 +848,17 @@ class Program(hildon.Program):
             try:
                 if icon_url:
                     icon_filename = uuid.uuid1().hex + urlparse.urlparse(icon_url)[2].split("/")[-1]
-                    urllib.urlretrieve(icon_url, os.path.join(create_config_dir(), icon_filename))
+                    urllib.urlretrieve(icon_url, os.path.join(self.__model.cache_path, icon_filename))
             except Exception, ex:
                 pass
-            feed_config =FeedConfig(url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0,
+            feed_config =FeedConfig(self.__model.cache_path, url, title, FeedConfig.mode_ommit, 7, "", "", False, icon_filename, 0,
                 False, "", "", "")
             edit_window = EditWindow(self.__window, feed_config)
             if edit_window.run() == gtk.RESPONSE_ACCEPT:
                 edit_window.update_feed()
                 self.__model.append(feed_config)
             elif icon_filename:
-                os.remove(os.path.join(create_config_dir(), icon_filename))
+                os.remove(os.path.join(self.__model.cache_path, icon_filename))
             edit_window.destroy()
         add_window.destroy()
 
@@ -832,24 +869,67 @@ class Program(hildon.Program):
         if len(selected):
             dialog = gtk.MessageDialog(self.__window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                 gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, _("Are you sure you want to delete selected feeds?"))
+            hbox = gtk.HBox()
+            button = gtk.CheckButton()
+            hbox.pack_start(button, expand = False)
+            hbox.pack_start(gtk.Label(_("Clear cache only")), expand = False)
+            hbox2 = gtk.HBox()
+            hbox2.pack_start(hbox, expand = True, fill = False)
+            hbox2.show_all()
+            dialog.vbox.pack_start(hbox2)
             response = dialog.run()
+            only_cache = button.get_active()
             dialog.destroy()
             if response == gtk.RESPONSE_YES:
                 for feed_config in selected:
-                    feed = feedcircuit.Feed(None, path = self.__config_path,
+                    feed = feedcircuit.Feed(None, path = self.__model.cache_path,
                         cache_path = ".cache", filename = feed_config.id + ".html")
                     feed.delete()
-                    if feed_config.icon_filename:
-                        icon_filepath = os.path.join(self.__config_path, feed_config.icon_filename)
-                        if os.path.exists(icon_filepath):
-                            os.remove(icon_filepath)
-                    self.__model.delete(feed_config.id)
+                    if not only_cache:
+                        if feed_config.icon_filename:
+                            icon_filepath = os.path.join(self.__model.cache_path, feed_config.icon_filename)
+                            if os.path.exists(icon_filepath):
+                                os.remove(icon_filepath)
+                        self.__model.delete(feed_config.id)
+
+
+    def move_cache(self, from_path, to_path):
+        to_path_cache = os.path.join(to_path, ".cache")
+        if not os.path.exists(to_path_cache):
+            os.makedirs(to_path_cache)
+        for file in glob.glob(os.path.join(from_path, "*")):
+            filename = os.path.split(file)[1]
+            if filename != "config":
+                os.rename(file, os.path.join(to_path, filename))
+        for file in glob.glob(os.path.join(from_path, ".cache/*")):
+            os.rename(file, os.path.join(to_path_cache, os.path.split(file)[1]))
+        os.rmdir(os.path.join(from_path, ".cache"))
+
+
+    def preferences(self, source):
+        pref_window = PreferencesWindow(self.__window)
+        try:
+            pref_window.path_entry.set_text(self.__model.cache_path)
+            if pref_window.run() == gtk.RESPONSE_ACCEPT:
+                new_path = os.path.abspath(pref_window.path_entry.get_text())
+                if new_path != self.__model.cache_path:
+                    self.move_cache(self.__model.cache_path, new_path)
+                    self.__model.update_cache_path(new_path)
+                self.__model.save_config()
+        except Exception, ex:
+            msg_dlg = gtk.MessageDialog(parent = self.__window, flags = gtk.DIALOG_MODAL,
+                type=gtk.MESSAGE_ERROR, buttons = gtk.BUTTONS_OK, 
+                message_format = str(ex))
+            msg_dlg.run()
+            msg_dlg.destroy()
+        finally:
+            pref_window.destroy()
 
 
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.4")
+        dialog.set_version("0.6.5")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
